package paradoxon;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.SwingConstants;
import javax.swing.ToolTipManager;

import org.opensourcephysics.display.BoundedImage;
import org.opensourcephysics.display.DrawingPanel;
import org.opensourcephysics.display.PlottingPanel;
import org.opensourcephysics.display.axes.CartesianType1;

public class ZwillingsHaupt extends JFrame {
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			@Override
			public void run() {
				try {
					ZwillingsHaupt fenster = new ZwillingsHaupt();
					fenster.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	public ZwillingsHaupt() {
		// Kopfzeile, Schließen, Größe (an Bildschirm angepasst),
		// GridBackLayout, Hintergrund
		super("Lehrprogramm zum relativistischen Dopplereffekt");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		setSize(screen);
		setVisible(true);
		JPanel contentPanel = new JPanel();
		GridBagLayout gbl = new GridBagLayout();
		contentPanel.setLayout(gbl);
		contentPanel.setBackground(new Color(231, 231, 231));

		// Schriftarten, Hintergrundfarbe, Buttongröße
		Font titelFont = new Font("Arial Unicode MS", Font.BOLD, 25);
		Font textFont = new Font("Arial Unicode MS", Font.PLAIN, 12);
		Font uebFont = new Font("Arial Unicode MS", Font.BOLD, 13);
		Color farbe = new Color(231, 231, 231);
		Dimension dimButton = new Dimension(110, 30);
		Dimension dimButton1 = new Dimension(50, 30);

		// Scrollen
		JScrollPane scroll = new JScrollPane(contentPanel);
		getContentPane().add(scroll);

		// Theorie Anfangswerte setzen
		ZwillingsRechnung zwillingsRechnung = new ZwillingsRechnung();
		zwillingsRechnung.setD(6);
		zwillingsRechnung.setV(.6);

		// Titel
		JLabel titelLabel = new JLabel("Das Zwillingsparadoxon");
		titelLabel.setFont(titelFont);

		// Panel für Plot des Raum-Zeit-Diagramms
		PlottingPanel plot = new PlottingPanel("Entfernung in Lichtjahren",
				"Vergangene Zeit seit dem Start", "Raum-Zeit-Diagramm");
		plot.setPreferredMinMax(-2, 20, -3, 35);
		plot.setAntialiasTextOn(true);
		plot.enableInspector(false);
		plot.setFont(textFont);
		plot.setBackground(farbe);

		String text1;
		text1 = "<html><b> Raum-Zeit-Diagramm: </b><br>"
				+ "<html> Die Zeit seit Beginn der Reise wird aufgetragen über dem Ort, "
				+ "der der Entfernung zur Erde entspricht. <br>"
				+ "<html> Das Koordinatensystem geh\u00f6rt also zum Inertialsystem, "
				+ "in dem der Erdzwilling (1) ruht. <br>"
				+ "<html> Zus\u00e4tzlich k\u00f6nnen die Koordinatensysteme "
				+ "eingeblendet werden, in denen der Raumfahrerzwilling (2)<br>"
				+ "<html>  w\u00e4hrend seiner Reise ruht. <br>"
				+ "<html> Als Orientierung ist der Lichtkegel in schwarz eingezeichnet. "
				+ "Er bleibt aufgrund der Konstanz der<br>"
				+ "<html>  Lichtgeschwindigkeit in jedem Koordinatensystem unver\u00e4ndert <br>"
				+ "<html>  <br>"
				+ "<html> Der Erdzwilling (1) sendet die gr\u00fcnen Lichtsignale aus,"
				+ " der Raumfahrerzwilling (2) sendet die <br>"
				+ "<html> orangefarbenen Lichtsignale aus.<br>"
				+ "<html> <br>"
				+ "<html> Die Weltlinie ist in zwei Farben aufgeteilt:<br>"
				+ "<html> W\u00e4hrend des roten Teils empfangen die Zwillinge die Signale rotverschoben, "
				+ "also mit geringerer Frequenz. <br>"
				+ "<html> W\u00e4hrend des blauen Teils empfangen die Zwillinge die Signale blauverschoben, "
				+ "also mit h\u00f6herer Frequenz.<br>"
				+ "<html> Der Dopplerfaktor zeigt jeweils an, um welchen Faktor sich "
				+ "die Freqzuenz ver\u00e4ndert"
				;
		plot.setToolTipText(text1);
		// Aktuelle Dauer holen:
		int dismissDelay = ToolTipManager.sharedInstance()
				.getDismissDelay();
		// Tooltip sichtbar halten:
		dismissDelay = Integer.MAX_VALUE;
		ToolTipManager.sharedInstance().setDismissDelay(dismissDelay);
		ToolTipManager.sharedInstance().registerComponent(plot);
		
		
		// Panel für Plot der Altersentwicklung
		PlottingPanel plotZeit = new PlottingPanel(" ",
				"Vergangene Zeit seit dem Start", "Alter der Zwillinge");
		plotZeit.setPreferredMinMax(0, 9, -3, 35);
		Dimension dim = new Dimension(100, 500);
		plotZeit.setPreferredSize(dim);
		plotZeit.setAntialiasTextOn(true);
		plotZeit.enableInspector(false);
		plotZeit.setFont(textFont);
		plotZeit.setBackground(farbe);
		// Bei der Altersentwicklung die Achsenbeschriftung der X-Achse
		// ausschalten
		CartesianType1 axes = (CartesianType1) plotZeit.getAxes();
		axes.addXTick("", 1.5);
		axes.addXTick("", 3.5);
		axes.addXTick("", 5.5);
		axes.addXTick("", 7.5);

		String text;
		text = "<html><b>Im Diagramm ''Alter der Zwillinge'': </b><br>"
				+ "<html><b>Gr\u00fcne Balken:</b> Altersentwicklung "
				+ "aus Sicht des ruhenden Zwillings (1). <br>"
				+ "<html><b>Orangefarbene Balken:</b> Altersentwicklung "
				+ "aus Sicht des reisenden Zwillings (2). <br>"
				+ "<html><b>jeweils links:</b> Alter des ruhenden Zwillings (1). <br>"
				+ "<html><b>jeweils rechts:</b> Alter des reisenden Zwillings (2). ";
		plotZeit.setToolTipText(text);
		// Aktuelle Dauer holen:
		int dismissDelay1 = ToolTipManager.sharedInstance()
				.getDismissDelay();
		// Tooltip sichtbar halten:
		dismissDelay1 = Integer.MAX_VALUE;
		ToolTipManager.sharedInstance().setDismissDelay(dismissDelay1);
		ToolTipManager.sharedInstance().registerComponent(plotZeit);
				
		// Start- und Reset-Button
		JButton startButton = new JButton("START");
		startButton.setFont(uebFont);
		startButton.setPreferredSize(dimButton);
		JButton resetButton = new JButton("RESET");
		resetButton.setPreferredSize(dimButton);
		resetButton.setFont(uebFont);
		// Nebeneinander angeordnet auf JPanel
		JPanel buttonsPanel = new JPanel();
		buttonsPanel.setLayout(new GridLayout(1, 2, 35, 10));
		buttonsPanel.setBackground(farbe);
		buttonsPanel.add(startButton);
		buttonsPanel.add(resetButton);

		// Schiebregler für Relgeschw und Entfernung

		// Schiebregler für Relativgeschwindigkeit:
		// Bereich von 50 bis 99 Prozent der Lichtgeschwindigkeit
		JSlider relgeschwSlider = new JSlider(SwingConstants.HORIZONTAL, 50,
				99, 60);
		relgeschwSlider.setMajorTickSpacing(5);
		relgeschwSlider.setMinorTickSpacing(1);
		relgeschwSlider.setPaintTicks(true);
		relgeschwSlider.setPaintLabels(true);
		relgeschwSlider.setSnapToTicks(true);
		relgeschwSlider.setBackground(farbe);
		// Label für Beschriftung des Schiebreglers Relativgeschwindigkeit
		JLabel relgeschwLabel = new JLabel(
				"Relativgeschwindigkeit in % von c: ");

		// Schiebregler für Entfernung
		// Bereich von 3 bis 8 Lichtjahren:
		// nicht kleiner als 3, weil das Bild sonst zu klein wird und nicht mehr
		// richtig erkannt werden kann
		// nicht größer als 8, weil sonst die kleinen Bilder nicht mehr sichtbar
		// sind
		// Voreinstellung 6: Bei 6 Lj und 0,6c kommen schöne Zahlen raus
		JSlider entfernungSlider = new JSlider(SwingConstants.HORIZONTAL, 3, 8,
				6);
		entfernungSlider.setMajorTickSpacing(1);
		entfernungSlider.setMinorTickSpacing(1);
		entfernungSlider.setPaintTicks(true);
		entfernungSlider.setPaintLabels(true);
		entfernungSlider.setSnapToTicks(true);
		entfernungSlider.setBackground(farbe);
		// Label für Beschriftung des Schiebreglers Entfernung
		JLabel entfernungLabel = new JLabel("Entfernung in Lichtjahren: ");
		// Extralabel für Abstand zwischen den Slidern
		JLabel abstandLabel = new JLabel(" ");
		// Regler und Beschriftung untereinander angeordnet auf JPanel
		JPanel sliderPanel = new JPanel();
		sliderPanel.setLayout(new BoxLayout(sliderPanel, BoxLayout.Y_AXIS));
		sliderPanel.setBackground(farbe);
		sliderPanel.add(relgeschwLabel);
		sliderPanel.add(relgeschwSlider);
		sliderPanel.add(abstandLabel);
		sliderPanel.add(entfernungLabel);
		sliderPanel.add(entfernungSlider);

		// Button für mehrere Bilder im Plot
		JButton mehrereBilderButton = new JButton("+");
		mehrereBilderButton.setFont(uebFont);
		mehrereBilderButton.setPreferredSize(dimButton1);
		mehrereBilderButton.setEnabled(false);
		JLabel mehrereBilderLabel1 = new JLabel("Zum Vergleichen ");
		mehrereBilderLabel1.setBackground(farbe);
		mehrereBilderLabel1.setFont(textFont);
		JLabel mehrereBilderLabel2 = new JLabel("mehrere Diagramme einzeichnen");
		mehrereBilderLabel2.setBackground(farbe);
		mehrereBilderLabel2.setFont(textFont);
		JPanel mehrereBilderLabelPanel = new JPanel();
		mehrereBilderLabelPanel.setBackground(farbe);
		mehrereBilderLabelPanel.setLayout(new BoxLayout(
				mehrereBilderLabelPanel, BoxLayout.Y_AXIS));
		mehrereBilderLabelPanel.add(mehrereBilderLabel1);
		mehrereBilderLabelPanel.add(mehrereBilderLabel2);
		// Extra Panel für Abstand zwischen den beiden Slidern
		JPanel abstandPanel = new JPanel();
		abstandPanel.setBackground(farbe);

		JPanel mehrereBilderPanel = new JPanel();
		mehrereBilderPanel.setBackground(farbe);
		mehrereBilderPanel.setLayout(new BoxLayout(mehrereBilderPanel,
				BoxLayout.X_AXIS));
		mehrereBilderPanel.add(mehrereBilderButton);
		mehrereBilderPanel.add(abstandPanel);
		mehrereBilderPanel.add(mehrereBilderLabelPanel);

		// Schiebregler für Bildrate
		JSlider bildrateSlider = new JSlider(SwingConstants.HORIZONTAL, 10,
				400, 100);
		bildrateSlider.setMajorTickSpacing(1);
		bildrateSlider.setMinorTickSpacing(1);
		bildrateSlider.setPaintTicks(false);
		bildrateSlider.setPaintLabels(false);
		bildrateSlider.setSnapToTicks(true);
		bildrateSlider.setBackground(farbe);
		// Label für Beschriftung des Schiebreglers Bildrate oben
		// (auf Panel für geschicktere Ausrichtung)
		// Bereich so angepasst, dass er gleich passt ohne Umrechnung
		JLabel bildrateLabelOben = new JLabel("Simulationsgeschwindigkeit: ");
		JPanel bildratePanelOben = new JPanel();
		bildratePanelOben.setLayout(new GridLayout(1, 1, 120, 10));
		bildratePanelOben.setBackground(farbe);
		bildratePanelOben.add(bildrateLabelOben);
		// Label für Beschriftung des Schiebreglers Bildrate unten
		// (auf dem Schiebregler-Panel für geschicktere Ausrichtung)
		JLabel bildrateLabelNiedrig = new JLabel("  langsam");
		JLabel bildrateLabelHoch = new JLabel("schnell");
		JPanel bildrateBeschriftungPanel = new JPanel();
		bildrateBeschriftungPanel.setLayout(new GridLayout(1, 2, 150, 10));
		bildrateBeschriftungPanel.setBackground(farbe);
		bildrateBeschriftungPanel.add(bildrateLabelNiedrig);
		bildrateBeschriftungPanel.add(bildrateLabelHoch);
		// Regler und Beschriftungen untereinander angeordnet auf JPanel
		JPanel bildratePanel = new JPanel();
		bildratePanel.setLayout(new BoxLayout(bildratePanel, BoxLayout.Y_AXIS));
		bildratePanel.setBackground(farbe);
		bildratePanel.add(bildratePanelOben);
		bildratePanel.add(bildrateSlider);
		bildratePanel.add(bildrateBeschriftungPanel);

		// Check-Boxen für Auswahl "Z2 sendet" und "Z1 sendet"
		JCheckBox checkboxZ2sendet = new JCheckBox("Zwilling 2 sendet", false);
		checkboxZ2sendet.setFont(textFont);
		checkboxZ2sendet.setBackground(farbe);
		JCheckBox checkboxZ1sendet = new JCheckBox("Zwilling 1 sendet", false);
		checkboxZ1sendet.setFont(textFont);
		checkboxZ1sendet.setBackground(farbe);

		// Bilder laden:
		// Zwilling 1 auf Erde
		BufferedImage zwilling1Datei = null; // leere Variable
		// leere Variable mit Bild füllen
		// try-catch, um Fehler beim Öffnen der Datei abzufangen
		try {
			zwilling1Datei = ImageIO.read(new File("Zwilling1.png"));
		} catch (IOException e) {
			e.printStackTrace();
		}
		// BoundedImage-Objekt erzeugen
		BoundedImage zwilling1 = new BoundedImage(zwilling1Datei, 0, 0);
		zwilling1.setWidth(zwilling1.getWidth() * 0.1);
		zwilling1.setHeight(zwilling1.getHeight() * 0.1);

		// Zwilling 2 auf Rakete, Rückweg
		BufferedImage zwilling2Datei = null; // leere Variable
		// leere Variable mit Bild füllen
		// try-catch, um Fehler beim Öffnen der Datei abzufangen
		try {
			zwilling2Datei = ImageIO.read(new File("Zwilling2Heimweg.png"));
		} catch (IOException e) {
			e.printStackTrace();
		}
		// BoundedImage-Objekt erzeugen
		BoundedImage zwilling2 = new BoundedImage(zwilling2Datei, 0, 0);
		zwilling2.setWidth(zwilling2.getWidth() * 0.1);
		zwilling2.setHeight(zwilling2.getHeight() * 0.1);
		// Panels für die Bilder
		Dimension dimImage = new Dimension(30, 160);
		DrawingPanel panelZw1 = new DrawingPanel();
		panelZw1.setBackground(farbe);
		panelZw1.setPreferredSize(dimImage);
		panelZw1.addDrawable(zwilling1);
		DrawingPanel panelZw2 = new DrawingPanel();
		panelZw2.setBackground(farbe);
		panelZw2.setPreferredSize(dimImage);
		panelZw2.addDrawable(zwilling2);
		// Anordnung: Zwilling und dazugehörige Checkbox übereinander
		// Zwilling 1 mit Checkbox 1
		JPanel zwilling1Panel = new JPanel();
		zwilling1Panel.setBackground(farbe);
		zwilling1Panel
				.setLayout(new BoxLayout(zwilling1Panel, BoxLayout.Y_AXIS));
		zwilling1Panel.add(panelZw1);
		zwilling1Panel.add(checkboxZ1sendet);
		// Zwilling 2 mit Checkbox 2
		JPanel zwilling2Panel = new JPanel();
		zwilling2Panel.setBackground(farbe);
		zwilling2Panel
				.setLayout(new BoxLayout(zwilling2Panel, BoxLayout.Y_AXIS));
		zwilling2Panel.add(panelZw2);
		zwilling2Panel.add(checkboxZ2sendet);
		// Panel mit Bild und Checkbox dann nebeneinander
		JPanel bilderCheckboxPanel = new JPanel();
		bilderCheckboxPanel.setBackground(farbe);
		bilderCheckboxPanel.setLayout(new BoxLayout(bilderCheckboxPanel,
				BoxLayout.X_AXIS));
		bilderCheckboxPanel.add(zwilling1Panel);
		bilderCheckboxPanel.add(zwilling2Panel);

		// Info-Button
		JButton infoButton = new JButton("INFO");
		infoButton.setFont(uebFont);
		infoButton.setPreferredSize(dimButton);
		JPanel infoButtonPanel = new JPanel();
		infoButtonPanel.setBackground(farbe);
		infoButtonPanel.setLayout(new BoxLayout(infoButtonPanel,
				BoxLayout.X_AXIS));
		infoButtonPanel.add(infoButton);
		infoButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				Infofenster infoFenster = new Infofenster(ZwillingsHaupt.this);
				infoFenster.setVisible(true);
			}
		});

		// Check-Box für Auswahl Achsen einbleinden
		JCheckBox checkboxVerloreneZeit = new JCheckBox(
				"Ereignisse, die aus Sicht des reisenden Zwillings", false);
		checkboxVerloreneZeit.setFont(textFont);
		checkboxVerloreneZeit.setBackground(farbe);
		JLabel verloreneZeitLabel = new JLabel(
				"       gleichzeitig zu seiner Umkehr stattfinden");
		verloreneZeitLabel.setFont(textFont);
		verloreneZeitLabel.setBackground(farbe);
		JCheckBox checkboxAchsen = new JCheckBox(
				"Koordinatenachsen Reiseteil 1 einblenden", false);
		checkboxAchsen.setFont(textFont);
		checkboxAchsen.setBackground(farbe);
		JCheckBox checkboxAchsen2 = new JCheckBox(
				"Koordinatenachsen Reiseteil 2 einblenden", false);
		checkboxAchsen2.setFont(textFont);
		checkboxAchsen2.setBackground(farbe);
		// Check-Box für Auswahl der Zwillings-Bilder
		JCheckBox checkboxBilder = new JCheckBox(
				"Bilder im Raum-Zeit-Diagramm einblenden", true);
		checkboxBilder.setFont(textFont);
		checkboxBilder.setBackground(farbe);
		// Checkbox für Nachskalieren
		JCheckBox checkboxSkalierung = new JCheckBox(
				"Raum-Zeit-Diagramm größtmöglich skalieren", false);
		checkboxSkalierung.setFont(textFont);
		checkboxSkalierung.setBackground(farbe);
		// Die beiden Checkboxen untereinander auf ein JPanel
		JPanel checkboxenUntenPanel = new JPanel();
		checkboxenUntenPanel.setLayout(new BoxLayout(checkboxenUntenPanel,
				BoxLayout.Y_AXIS));
		checkboxenUntenPanel.setBackground(farbe);
		checkboxenUntenPanel.add(checkboxVerloreneZeit);
		checkboxenUntenPanel.add(verloreneZeitLabel);
		checkboxenUntenPanel.add(checkboxAchsen);
		checkboxenUntenPanel.add(checkboxAchsen2);
		checkboxenUntenPanel.add(checkboxBilder);
		checkboxenUntenPanel.add(checkboxSkalierung);

		// Komponenten auf contentPanel hinzufügen
		// (Gridbag-Layout verwendet)

		// Titel oben, zentriert (0|0)
		GridBagConstraints gbTitel = new GridBagConstraints();
		gbTitel.gridx = 0;
		gbTitel.gridy = 0;
		gbTitel.gridwidth = 3;
		gbTitel.gridheight = 1;
		gbTitel.anchor = GridBagConstraints.CENTER;
		gbTitel.insets = new Insets(15, 0, 20, 0);
		gbl.setConstraints(titelLabel, gbTitel);
		contentPanel.add(titelLabel);

		// Plot Raum-Zeit-Diagramm, unter Titel links (0|1)
		GridBagConstraints gbPlot = new GridBagConstraints();
		gbPlot.gridx = 0;
		gbPlot.gridy = 1;
		gbPlot.gridwidth = 1;
		gbPlot.gridheight = 9;
		gbPlot.weightx = 100;
		gbPlot.weighty = 100;
		gbPlot.insets = new Insets(5, 0, 5, 0);
		gbPlot.fill = GridBagConstraints.BOTH;
		gbPlot.anchor = GridBagConstraints.CENTER;
		gbl.setConstraints(plot, gbPlot);
		contentPanel.add(plot);

		// Plot Altersentwicklung Zwillinge, unter Titel mittig (1|1)
		GridBagConstraints gbPlotZeit = new GridBagConstraints();
		gbPlotZeit.gridx = 1;
		gbPlotZeit.gridy = 1;
		gbPlotZeit.gridwidth = 1;
		gbPlotZeit.gridheight = 9;
		gbPlotZeit.weightx = 100;
		gbPlotZeit.weighty = 100;
		gbPlotZeit.insets = new Insets(5, 0, 5, 0);
		gbPlotZeit.fill = GridBagConstraints.BOTH;
		gbPlotZeit.anchor = GridBagConstraints.CENTER;
		gbl.setConstraints(plotZeit, gbPlotZeit);
		contentPanel.add(plotZeit);

		// Restliche Komponenten werden in 3. Spalte, also neben dem Altersplot
		// untereinander angeordnet. (2|1,2,3,...)
		// Bilder der Zwillinge und Checkboxen ganz oben (2|1)
		GridBagConstraints gbBilderCheckbox = new GridBagConstraints();
		gbBilderCheckbox.gridx = 2;
		gbBilderCheckbox.gridy = 1;
		gbBilderCheckbox.gridwidth = 1;
		gbBilderCheckbox.gridheight = 1;
		gbBilderCheckbox.insets = new Insets(0, 15, 5, 15);
		gbBilderCheckbox.anchor = GridBagConstraints.CENTER;
		gbl.setConstraints(bilderCheckboxPanel, gbBilderCheckbox);
		contentPanel.add(bilderCheckboxPanel);

		// Schiebregler Geschwindigkeit mit etwas Abstand als drittes (2|2)
		// zusammen mit Schiebregler Entfernung (unter Schiebregler Geschw)
		GridBagConstraints gbSliderGeschw = new GridBagConstraints();
		gbSliderGeschw.gridx = 2;
		gbSliderGeschw.gridy = 2;
		gbSliderGeschw.gridwidth = 1;
		gbSliderGeschw.gridheight = 1;
		gbSliderGeschw.insets = new Insets(30, 15, 5, 15);
		gbSliderGeschw.anchor = GridBagConstraints.CENTER;
		gbl.setConstraints(sliderPanel, gbSliderGeschw);
		contentPanel.add(sliderPanel);

		// Button um mehrere Bilder zu vergleichen
		GridBagConstraints gbMehrereBilder = new GridBagConstraints();
		gbMehrereBilder.gridx = 2;
		gbMehrereBilder.gridy = 3;
		gbMehrereBilder.gridwidth = 1;
		gbMehrereBilder.gridheight = 1;
		gbMehrereBilder.insets = new Insets(10, 15, 5, 15);
		gbMehrereBilder.anchor = GridBagConstraints.CENTER;
		gbl.setConstraints(mehrereBilderPanel, gbMehrereBilder);
		contentPanel.add(mehrereBilderPanel);

		// Extra Panel (untenPanel) für die unteren drei Komponenten
		// Mit Gridbaglayout
		JPanel untenPanel = new JPanel();
		untenPanel.setBackground(farbe);
		GridBagLayout gblu = new GridBagLayout();
		untenPanel.setLayout(gblu);
		untenPanel.setBackground(farbe);
		// Info-Button
		GridBagConstraints gbInfoButton = new GridBagConstraints();
		gbInfoButton.gridx = 0;
		gbInfoButton.gridy = 4;
		gbInfoButton.gridwidth = 1;
		gbInfoButton.gridheight = 1;
		gbInfoButton.insets = new Insets(15, 0, 0, 0);
		gbInfoButton.anchor = GridBagConstraints.EAST;
		gblu.setConstraints(infoButtonPanel, gbInfoButton);
		untenPanel.add(infoButtonPanel);
		// Checkbox für Bilder und Nachskalieren
		GridBagConstraints gbCheckBoxBilderSkalierung = new GridBagConstraints();
		gbCheckBoxBilderSkalierung.gridx = 0;
		gbCheckBoxBilderSkalierung.gridy = 1;
		gbCheckBoxBilderSkalierung.gridwidth = 1;
		gbCheckBoxBilderSkalierung.gridheight = 1;
		gbCheckBoxBilderSkalierung.insets = new Insets(0, 5, 5, 15);
		gbCheckBoxBilderSkalierung.anchor = GridBagConstraints.CENTER;
		gblu.setConstraints(checkboxenUntenPanel, gbCheckBoxBilderSkalierung);
		untenPanel.add(checkboxenUntenPanel);
		// Start- und Reset-Buttons
		GridBagConstraints gbButtons = new GridBagConstraints();
		gbButtons.gridx = 0;
		gbButtons.gridy = 2;
		gbButtons.gridwidth = 1;
		gbButtons.gridheight = 1;
		gbButtons.insets = new Insets(15, 0, 5, 15);
		gbButtons.anchor = GridBagConstraints.CENTER;
		gblu.setConstraints(buttonsPanel, gbButtons);
		untenPanel.add(buttonsPanel);
		// Schiebregler Bildrate
		GridBagConstraints gbBildrate = new GridBagConstraints();
		gbBildrate.gridx = 0;
		gbBildrate.gridy = 3;
		gbBildrate.gridwidth = 1;
		gbBildrate.gridheight = 1;
		gbBildrate.insets = new Insets(15, 5, 5, 15);
		gbBildrate.anchor = GridBagConstraints.CENTER;
		gblu.setConstraints(bildratePanel, gbBildrate);
		untenPanel.add(bildratePanel);

		// Extra untenPanel auf contentPanel hinzufügen
		GridBagConstraints gbUntereKomponenten = new GridBagConstraints();
		gbUntereKomponenten.gridx = 2;
		gbUntereKomponenten.gridy = 9;
		gbUntereKomponenten.gridwidth = 1;
		gbUntereKomponenten.gridheight = 1;
		gbUntereKomponenten.insets = new Insets(5, 15, 25, 15);
		gbUntereKomponenten.anchor = GridBagConstraints.SOUTH;
		gbl.setConstraints(untenPanel, gbUntereKomponenten);
		contentPanel.add(untenPanel);

		new ZwillingsSim(plot, plotZeit, zwillingsRechnung, startButton,
				resetButton, relgeschwSlider, entfernungSlider, bildrateSlider,
				checkboxZ2sendet, checkboxZ1sendet, checkboxBilder,
				checkboxSkalierung, checkboxAchsen, checkboxAchsen2,
				mehrereBilderButton, checkboxVerloreneZeit, verloreneZeitLabel);
	}

}

/*
 * This software is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License (GPL) as published by the
 * Free Software Foundation; either version 3 of the License, or(at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this; if not, see <http://www.gnu.org.licenses>
 */