package paradoxon;

public class ZwillingsRechnung {
	double v; // Relativgeschwindigkeit in Einheiten von c
	double d; // Entfernung zum Umkehrpunkt in Lichtjahren

	// Relativgeschwindigkeit von aussen einlesen
	public void setV(double v) {
		this.v = v;
	}

	// Entfernung zum Umkehrpunkt von aussen einlesen
	public void setD(double d) {
		this.d = d;
	}

	public double getV() {
		return v;
	}

	public double getD() {
		return d;
	}

	// Gesamtreisezeit in I1 (in Jahren)
	public double getGesT() {
		double gesT = 2 * d / v;
		return gesT;
	}

	// Berechnung der Position von Z2
	public double getPosZ2(double t) {
		double ret;
		if (t <= getGesT() / 2) {
			ret = v * t;
		} else if (t > getGesT() / 2) {
			ret = -v * t + 2 * d;
		} else {
			ret = 0;
		}
		return ret;
	}

	// Zeitdilatation
	public double getZeitdilatationZ2(double t) {
		double ret;
		if (t <= getGesT()) {
			ret = t / Math.sqrt(1 - v * v);
		} else {
			ret = t;
		}
		return ret;
	}

	// Lichtsignale, die der ruhende Zwilling (Z1) aussendet
	public double[] getLichtsignalZ1(double t) {
		double[] ret = new double[(int) getGesT()];
		for (int i = 1; i < getGesT(); i++) {
			if ((i <= t) && ((i + getPosZ2(t)) >= t)) {
				ret[i - 1] = t - i;
			}
		}
		return ret;
	}

	// Lichtsignale, die der bewegte Zwilling (Z2) aussendet
	public double[] getLichtsignalZ2(double t) {
		double[] ret = new double[(int) getGesT()];
		for (int j = 1; j < getGesT(); j++) {
			if ((getZeitdilatationZ2(j) <= t)
					&& ((getZeitdilatationZ2(j) + getPosZ2(getZeitdilatationZ2(j))) >= t)) {
				ret[j - 1] = -t + getZeitdilatationZ2(j)
						+ getPosZ2(getZeitdilatationZ2(j));
			}
		}
		return ret;
	}

	// Dopplerfaktor während der Hinreise
	public double getDopplerfaktorHin() {
		double ret = Math.sqrt((1 - v) / (1 + v));
		return ret;
	}

	// Dopplerfaktor während der Rückreise
	public double getDopplerfaktorRueck() {
		double ret = Math.sqrt((1 + v) / (1 - v));
		return ret;
	}

	// Zeitpunkt an dem Z1 von der Umkehr von Z2 erfährt (in Jahren)
	// (Benötigt für Altersberechnung)
	public double getT1() {
		double t1 = getGesT() / 2 + d;
		return t1;
	}

	// Alter von Z2 aus Sicht von Z1, eigenes Alter ist einfach t
	public double getAlterZ2SichtZ1(double t) {
		double ret;
		if (t <= getT1()) {
			ret = t * getDopplerfaktorHin();
		} else if (t > getT1()) {
			ret = (t - getT1()) * getDopplerfaktorRueck() + getT1() * getDopplerfaktorHin();
		} else {
			ret = 0;
		}
		return ret;
	}

	// Alter von Z2 aus Sicht von Z2
	public double getAlterZ2SichtZ2(double t) {
		double ret;
		if (t <= getGesT()) {
			ret = t * Math.sqrt(1 - v * v);
		} else {
			ret = 0;
		}
		return ret;
	}

	// Alter von Z1 aus Sicht von Z2
	public double getAlterZ1SichtZ2(double t) {
		double ret;
		if (t <= getGesT() / 2) {
			ret = getAlterZ2SichtZ2(t) * getDopplerfaktorHin();
		} else if (t > getGesT() / 2) {
			ret = (getAlterZ2SichtZ2(t) - getAlterZ2SichtZ2(getGesT() / 2) ) * getDopplerfaktorRueck()
					+ getAlterZ2SichtZ2(getGesT() / 2) * getDopplerfaktorHin();
		} else {
			ret = 0;
		}
		return ret;
	}
}

/*
 * This software is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License (GPL) as published by the
 * Free Software Foundation; either version 3 of the License, or(at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this; if not, see <http://www.gnu.org.licenses>
 */