package paradoxon;

import java.awt.Color;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import javax.imageio.ImageIO;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.opensourcephysics.display.BoundedImage;
import org.opensourcephysics.display.Dataset;
import org.opensourcephysics.display.PlottingPanel;

public class ZwillingsSim implements Runnable {

	ScheduledExecutorService executor;

	double t = 0;
	double dt = 0.005;

	// Simulationsrate
	double rate = 60;
	// Verzögerung für flüssiges Bild
	long delay = (long) (0.3E6 / rate);
	// PlottingPanels vordefinieren
	PlottingPanel plot;
	PlottingPanel plotZeit;
	TextImPlot dopplerTeil1;
	TextImPlot dopplerTeil2;
	TextImPlot textZ1;
	TextImPlot textZ2;
	// Objekte der Rechnung-Klasse vordefinieren
	ZwillingsRechnung zwillingsRechnung;
	// Start-, Pause-, Schritt-, Reset-Button vordefinieren
	JButton startButton;
	JButton resetButton;
	JButton mehrerBilderButton;
	// Schiebregler vordefinieren
	JSlider relgeschwSlider;
	JSlider entfernungSlider;
	JSlider bildrateSlider;
	// Check-Boxen vordefinieren
	JCheckBox checkboxZ2sendet;
	JCheckBox checkboxZ1sendet;
	JCheckBox checkboxBilder;
	JCheckBox checkboxSkalierung;
	JCheckBox checkboxAchsen;
	JCheckBox checkboxAchsen2;
	JCheckBox checkboxVerloreneZeit;
	// Label für die zweite Zeile der Checkbox vordefinieren
	JLabel verloreneZeitLabel;
	// Datasets des Plots vordefinieren
	Dataset zwilling1Data;
	Dataset zwilling1aData;
	Dataset zwilling2Data;
	Dataset zwilling2aData;
	Dataset signal1Data;
	Dataset signal2Data;
	// Rechtecke als Balkendiagramme
	Rechteck rechteck1;
	Rechteck rechteck2;
	Rechteck rechteck3;
	Rechteck rechteck4;
	// Dreieck für verlorene Zeit
	VerloreneZeit verloreneZeit;
	// Linie für Lichtkegel
	Lichtkegel lichtkegel;
	AchsenBewegtesIS achsenBewegtesIS;
	AchsenBewegtesIS2 achsenBewegtesIS2;
	// booelan für Start-/Stop-Knopf vordefinieren
	boolean sim = false;
	// boolean für Umschalten auf Rückweg
	boolean rueck = false;
	// boolean für Ein-/Ausgeblendete Bilder
	boolean bilder = true;
	// boolean für Skalierung
	boolean skalierung = false;
	// Bilder als Klasse BoundedImage anlegen
	private BoundedImage zwilling1;
	private BoundedImage zwilling2Hin;
	private BoundedImage zwilling2Rueck;
	// Labels für die Dopplerfrequenz
	JLabel dopplerfrequenz1Label;
	JLabel dopplerfrequenz2Label;

	public ZwillingsSim(final PlottingPanel plot, final PlottingPanel plotZeit,
			final ZwillingsRechnung zwillingsRechnung,
			final JButton startButton, final JButton resetButton,
			final JSlider relgeschwSlider, final JSlider entfernungSlider,
			final JSlider bildrateSlider, final JCheckBox checkboxZ2sendet,
			final JCheckBox checkboxZ1sendet, final JCheckBox checkboxBilder,
			final JCheckBox checkboxSkalierung, final JCheckBox checkboxAchsen,
			final JCheckBox checkboxAchsen2, final JButton mehrereBilderButton,
			final JCheckBox checkboxVerloreneZeit, final JLabel verloreneZeitLabel) {
		// Übergebene Objekte mit den vordefinierten Objekten identifizieren
		this.plot = plot;
		this.plotZeit = plotZeit;
		this.zwillingsRechnung = zwillingsRechnung;
		this.startButton = startButton;
		this.resetButton = resetButton;
		this.relgeschwSlider = relgeschwSlider;
		this.entfernungSlider = entfernungSlider;
		this.bildrateSlider = bildrateSlider;
		this.checkboxZ2sendet = checkboxZ2sendet;
		this.checkboxZ1sendet = checkboxZ1sendet;
		this.checkboxBilder = checkboxBilder;
		this.checkboxSkalierung = checkboxSkalierung;
		this.checkboxAchsen = checkboxAchsen;
		this.checkboxAchsen2 = checkboxAchsen2;
		this.checkboxVerloreneZeit = checkboxVerloreneZeit;
		this.mehrerBilderButton = mehrereBilderButton;
		this.verloreneZeitLabel = verloreneZeitLabel;

		// aeussere Linie fuer ruhenden Zwilling in rot und blau
		zwilling1Data = new Dataset();
		zwilling1Data.setMarkerColor(Color.RED);
		zwilling1Data.setMarkerShape(Dataset.PIXEL);
		zwilling1Data.setConnected(false);
		zwilling1Data.setMaximumPoints(50000);
		plot.addDrawable(zwilling1Data);
		zwilling1aData = new Dataset();
		zwilling1aData.setMarkerColor(Color.BLUE);
		zwilling1aData.setMarkerShape(Dataset.PIXEL);
		zwilling1aData.setConnected(false);
		zwilling1aData.setMaximumPoints(50000);
		plot.addDrawable(zwilling1aData);

		// aeussere Linie fuer reisenden Zwilling in rot und blau
		zwilling2Data = new Dataset();
		zwilling2Data.setMarkerColor(Color.RED);
		zwilling2Data.setMarkerShape(Dataset.PIXEL);
		zwilling2Data.setConnected(false);
		zwilling2Data.setMaximumPoints(50000);
		plot.addDrawable(zwilling2Data);
		zwilling2aData = new Dataset();
		zwilling2aData.setMarkerColor(Color.BLUE);
		zwilling2aData.setMarkerShape(Dataset.PIXEL);
		zwilling2aData.setConnected(false);
		zwilling2aData.setMaximumPoints(50000);
		plot.addDrawable(zwilling2aData);

		// Signale vom ruhenden Zwilling in gruen
		signal1Data = new Dataset();
		signal1Data.setMarkerShape(Dataset.PIXEL);
		signal1Data.setMarkerColor(Color.GREEN);
		signal1Data.setConnected(false);
		signal1Data.setMaximumPoints(50000);

		// Signale vom reisenden Zwilling in orange
		signal2Data = new Dataset();
		signal2Data.setMarkerShape(Dataset.PIXEL);
		signal2Data.setMarkerColor(Color.ORANGE);
		signal2Data.setConnected(false);
		signal2Data.setMaximumPoints(50000);

		// Rechtecke für Balkendiagramm werden auf x-Achse positioniert und
		// Farbe wird zugeordnet
		rechteck1 = new Rechteck(0, 1, Color.GREEN);
		rechteck2 = new Rechteck(0, 3, Color.GREEN);
		rechteck3 = new Rechteck(0, 5, Color.ORANGE);
		rechteck4 = new Rechteck(0, 7, Color.ORANGE);
		plotZeit.addDrawable(rechteck1);
		plotZeit.addDrawable(rechteck2);
		plotZeit.addDrawable(rechteck3);
		plotZeit.addDrawable(rechteck4);

		// Dreieck für verlorene Zeit
		verloreneZeit = new VerloreneZeit();
		verloreneZeit.setV(0.6);
		verloreneZeit.setD(6);
		
		// Linie des Lichtkegels zeichnen
		lichtkegel = new Lichtkegel();
		plot.addDrawableAtIndex(0, lichtkegel);
		
		// Achsen des bewegten Koordinatensystems1
		achsenBewegtesIS = new AchsenBewegtesIS();
		achsenBewegtesIS.setV(0.6);

		// Achsen des bewegten Koordinatensystems2
		achsenBewegtesIS2 = new AchsenBewegtesIS2();
		achsenBewegtesIS2.setV(0.6);
		achsenBewegtesIS2.setD(6);
		
		// CheckBox für Ein-/Ausblenden der Zwillings-Bilder
		checkboxBilder.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					// Bild Zwilling1 hinzufügen
					zwilling1.setXY(0, t);
					plot.addDrawable(zwilling1);
					// Hinzufügen: Hinflug -> Zwilling2Hin, Rückflug ->
					// Zwilling2Rück
					if (t < zwillingsRechnung.getGesT() / 2) {
						zwilling2Hin.setXY(zwillingsRechnung.getPosZ2(t), t);
						plot.addDrawable(zwilling2Hin);
						rueck = false;
					} else {
						zwilling2Rueck.setXY(zwillingsRechnung.getPosZ2(t), t);
						plot.addDrawable(zwilling2Rueck);
					}
					plot.repaint();
					bilder = true;
				} else if (e.getStateChange() == ItemEvent.DESELECTED) {
					plot.removeDrawable(zwilling1);
					plot.removeDrawable(zwilling2Hin);
					plot.removeDrawable(zwilling2Rueck);
					plot.repaint();
					rueck = true;
					bilder = false;
				}
			}
		});

		// CheckBox2 für Achsen
		checkboxVerloreneZeit.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					plot.addDrawable(verloreneZeit);
					plot.repaint();
				} else if (e.getStateChange() == ItemEvent.DESELECTED) {
					plot.removeDrawable(verloreneZeit);
					plot.repaint();
				}
			}
		});
		
		// CheckBox1 für Achsen
		checkboxAchsen.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					plot.addDrawableAtIndex(0, achsenBewegtesIS);
					plot.repaint();
				} else if (e.getStateChange() == ItemEvent.DESELECTED) {
					plot.removeDrawable(achsenBewegtesIS);
					plot.repaint();
				}
			}
		});
		// CheckBox2 für Achsen
		checkboxAchsen2.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					plot.addDrawableAtIndex(0, achsenBewegtesIS2);
					plot.repaint();
				} else if (e.getStateChange() == ItemEvent.DESELECTED) {
					plot.removeDrawable(achsenBewegtesIS2);
					plot.repaint();
				}
			}
		});
		// CheckBox für Nachskalierung
		checkboxSkalierung.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					plot.setPreferredMinMax(-.15 * zwillingsRechnung.getD(),
							1.15 * zwillingsRechnung.getD(), -.1
									* zwillingsRechnung.getGesT(),
							1.1 * zwillingsRechnung.getGesT());
					plotZeit.setPreferredMinMaxY(
							-.1 * zwillingsRechnung.getGesT(),
							1.1 * zwillingsRechnung.getGesT());
					skalierung = true;
					plot.repaint();
					plotZeit.repaint();
				} else if (e.getStateChange() == ItemEvent.DESELECTED) {
					plot.setPreferredMinMax(-2, 19, -3, 35);
					plotZeit.setPreferredMinMax(0, 9, -3, 35);
					skalierung = false;
					plot.repaint();
					plotZeit.repaint();
				}
			}
		});

		// CheckBox für Auswahl von Sendesignalen
		checkboxZ2sendet.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					plot.addDrawableAtIndex(1, signal2Data);
					plot.repaint();
				} else if (e.getStateChange() == ItemEvent.DESELECTED) {
					plot.removeDrawable(signal2Data);
					plot.repaint();
				}
			}
		});
		checkboxZ1sendet.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					plot.addDrawableAtIndex(1, signal1Data);
					plot.repaint();
				} else if (e.getStateChange() == ItemEvent.DESELECTED) {
					plot.removeDrawable(signal1Data);
					plot.repaint();
				}
			}
		});

		// Schieberegler für Relativgeschwindigkeit anschliessen
		relgeschwSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent event) {
				JSlider source;
				source = (JSlider) event.getSource();
				if (!source.getValueIsAdjusting()) {
					zwillingsRechnung.setV(((double) source.getValue()) / 100);
					dopplerTeil1.setZahl(zwillingsRechnung.getDopplerfaktorHin());
					dopplerTeil2.setZahl(zwillingsRechnung.getDopplerfaktorRueck());
					dopplerTeil2.setY(zwillingsRechnung.getGesT()-0.5);
					achsenBewegtesIS.setV(zwillingsRechnung.getV());
					achsenBewegtesIS2.setV(zwillingsRechnung.getV());
					verloreneZeit.setV(zwillingsRechnung.getV());
					if (skalierung) {
						plot.setPreferredMinMax(
								-.15 * zwillingsRechnung.getD(),
								1.15 * zwillingsRechnung.getD(), -.1
										* zwillingsRechnung.getGesT(),
								1.1 * zwillingsRechnung.getGesT());
						plotZeit.setPreferredMinMaxY(
								-.1 * zwillingsRechnung.getGesT(),
								1.1 * zwillingsRechnung.getGesT());
						skalierung = true;
						plot.repaint();
						plotZeit.repaint();
					} else {
						plot.setPreferredMinMax(-2, 20, -3, 35);
						plotZeit.setPreferredMinMax(0, 9, -3, 35);
						plot.repaint();
						plotZeit.repaint();
					}
				}
			}
		});

		// Schiebregler für Entfernung anschliessen
		entfernungSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent event) {
				JSlider source;
				source = (JSlider) event.getSource();
				if (!source.getValueIsAdjusting()) {
					zwillingsRechnung.setD(source.getValue());
					dopplerTeil1.setX(zwillingsRechnung.getD()/2.5);
					dopplerTeil2.setY(zwillingsRechnung.getGesT()-0.5);
					dopplerTeil2.setX(zwillingsRechnung.getD()/2.5);
					achsenBewegtesIS2.setD(zwillingsRechnung.getD());
					verloreneZeit.setD(zwillingsRechnung.getD());
					if (skalierung) {
						plot.setPreferredMinMax(
								-.15 * zwillingsRechnung.getD(),
								1.15 * zwillingsRechnung.getD(), -.1
										* zwillingsRechnung.getGesT(),
								1.1 * zwillingsRechnung.getGesT());
						plotZeit.setPreferredMinMaxY(
								-.1 * zwillingsRechnung.getGesT(),
								1.1 * zwillingsRechnung.getGesT());
						skalierung = true;
						plot.repaint();
						plotZeit.repaint();

					} else {
						plot.setPreferredMinMax(-2, 20, -3, 35);
						plotZeit.setPreferredMinMax(0, 9, -3, 35);
						plot.repaint();
						plotZeit.repaint();
					}
				}
			}
		});

		// Texte im PlottingPanel
		dopplerTeil1 = new TextImPlot("Dopplerfaktor Teil 1 der Reise: ",
				zwillingsRechnung.getDopplerfaktorHin(), zwillingsRechnung.getD()/2.5, 
				0.5, Color.RED);
		dopplerTeil2 = new TextImPlot("Dopplerfaktor Teil 2 der Reise: ",
				zwillingsRechnung.getDopplerfaktorRueck(), zwillingsRechnung.getD()/2.5, 
				zwillingsRechnung.getGesT()-0.5, Color.BLUE);
		plot.addDrawable(dopplerTeil1);
		plot.addDrawable(dopplerTeil2);
		
		// Slider für Bildrate: Bildrate wird neu gesetzt
		bildrateSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent event) {
				JSlider source;
				source = (JSlider) event.getSource();
				if (!source.getValueIsAdjusting()) {
					if (!sim) {
						rate = (double) source.getValue();
						delay = (long) (0.3E6 / rate);
					} else {
						executor.shutdown();
						rate = (double) source.getValue();
						delay = (long) (0.3E6 / rate);
						executor = Executors.newScheduledThreadPool(1);
						executor.scheduleAtFixedRate(ZwillingsSim.this, delay,
								delay, TimeUnit.MICROSECONDS);
					}
				}

			}
		});

		// Start-/Stop-Button anschliessen
		startButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (!sim) {
					// executor starten
					executor = Executors.newScheduledThreadPool(1);
					executor.scheduleAtFixedRate(ZwillingsSim.this, delay,
							delay, TimeUnit.MICROSECONDS);
					// boolean umstellen und Buttontext ändern nachdem
					// gedrückt wurde
					sim = true;
					ZwillingsSim.this.startButton.setText("PAUSE");
					relgeschwSlider.setEnabled(false);
					entfernungSlider.setEnabled(false);
				} else {
					// executor beenden
					executor.shutdown();
					// boolean umstellen und Buttontext ändern nachdem
					// gedrückt wurde
					sim = false;
					ZwillingsSim.this.startButton.setText("WEITER");
				}
			}
		});

		// Reset-Button anschliessen
		resetButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				// executor beenden
				executor.shutdown();
				// beim Start-/Stop-Button Text umstellen und boolean umstellen
				ZwillingsSim.this.startButton.setText("START");
				sim = false;
				// Anfangswerte zurücksetzen
				t = 0;

				// DataSets leeren
				zwilling1Data.clear();
				zwilling2Data.clear();
				zwilling1aData.clear();
				zwilling2aData.clear();
				signal1Data.clear();
				signal2Data.clear();
				// Balken zurücksetzen
				rechteck1.setHoehe(0);
				rechteck2.setHoehe(0);
				rechteck3.setHoehe(0);
				rechteck4.setHoehe(0);

				// Zwillings Bilder auf Anfangssituation zurücksetzen falls
				// CheckBoxBilder ausgewählt
				if (bilder) {
					zwilling1.setXY(0, 0);
					// Zurücksetzen je nach Flugphase
					if (rueck) {
						plot.removeDrawable(zwilling2Rueck);
						zwilling2Hin.setXY(0, 0);
						plot.addDrawable(zwilling2Hin);
						rueck = false;
					} else {
						zwilling2Hin.setXY(0, 0);
					}
				}

				// Auswahl freischalten
				relgeschwSlider.setEnabled(true);
				entfernungSlider.setEnabled(true);
				startButton.setEnabled(true);
				checkboxSkalierung.setEnabled(true);
				checkboxAchsen.setEnabled(true);
				checkboxAchsen2.setEnabled(true);
				checkboxZ1sendet.setEnabled(true);
				checkboxZ2sendet.setEnabled(true);
				checkboxVerloreneZeit.setEnabled(true);
				verloreneZeitLabel.setForeground(Color.black);
				// mehrere Bilder-Button sperren
				mehrerBilderButton.setEnabled(false);
				plot.addDrawable(dopplerTeil1);
				plot.addDrawable(dopplerTeil2);
				plot.repaint();
				plotZeit.repaint();
			}
		});

		// MehrereBilder-Button anschliessen
				mehrereBilderButton.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent e) {
						// executor beenden
						executor.shutdown();
						// beim Start-/Stop-Button Text umstellen und boolean umstellen
						ZwillingsSim.this.startButton.setText("START");
						sim = false;
						// Anfangswerte zurücksetzen
						t = 0;

						// DataSets für Signale leeren, andere lassen
						signal1Data.clear();
						signal2Data.clear();
						// Balken zurücksetzen
						rechteck1.setHoehe(0);
						rechteck2.setHoehe(0);
						rechteck3.setHoehe(0);
						rechteck4.setHoehe(0);

						// Zwillings Bilder auf Anfangssituation zurücksetzen falls
						// CheckBoxBilder ausgewählt
						if (bilder) {
							zwilling1.setXY(0, 0);
							// Zurücksetzen je nach Flugphase
							if (rueck) {
								plot.removeDrawable(zwilling2Rueck);
								zwilling2Hin.setXY(0, 0);
								plot.addDrawable(zwilling2Hin);
								rueck = false;
							} else {
								zwilling2Hin.setXY(0, 0);
							}
						}

						// Auswahl anpassen
						relgeschwSlider.setEnabled(true);
						entfernungSlider.setEnabled(true);
						startButton.setEnabled(true);
						checkboxSkalierung.setSelected(false);
						checkboxAchsen.setSelected(false);
						checkboxAchsen2.setSelected(false);
						checkboxVerloreneZeit.setSelected(false);
						checkboxZ1sendet.setSelected(false);
						checkboxZ2sendet.setSelected(false);
						checkboxSkalierung.setEnabled(false);
						checkboxAchsen.setEnabled(false);
						checkboxAchsen2.setEnabled(false);
						checkboxVerloreneZeit.setEnabled(false);
						verloreneZeitLabel.setForeground(Color.GRAY);
						checkboxZ1sendet.setEnabled(false);
						checkboxZ2sendet.setEnabled(false);
						mehrerBilderButton.setEnabled(false);
						plot.removeDrawable(dopplerTeil1);
						plot.removeDrawable(dopplerTeil2);
						plot.repaint();
						plotZeit.repaint();						
					}
				});

		
		// Bilder laden:

		// Zwilling 1 auf Erde
		BufferedImage zwilling1Datei = null; // leere Variable
		// leere Variable mit Bild füllen
		// try-catch, um Fehler beim Öffnen der Datei abzufangen
		try {
			zwilling1Datei = ImageIO.read(new File("Zwilling1.png"));
		} catch (IOException e) {
			e.printStackTrace();
		}
		// BoundedImage-Objekt erzeugen
		zwilling1 = new BoundedImage(zwilling1Datei, 0, 0);
		zwilling1.setWidth(zwilling1.getWidth() * 0.08);
		zwilling1.setHeight(zwilling1.getHeight() * 0.08);
		// Bild von Zwilling1 einfügen
		plot.addDrawable(zwilling1);

		// Zwilling 2 auf Rakete, Hinweg
		BufferedImage zwilling2HinDatei = null; // leere Variable
		// leere Variable mit Bild füllen
		// try-catch, um Fehler beim Öffnen der Datei abzufangen
		try {
			zwilling2HinDatei = ImageIO.read(new File("Zwilling2Hinweg.png"));
		} catch (IOException e) {
			e.printStackTrace();
		}
		// BoundedImage-Objekt erzeugen
		zwilling2Hin = new BoundedImage(zwilling2HinDatei, 0, 0);
		zwilling2Hin.setWidth(zwilling2Hin.getWidth() * 0.08);
		zwilling2Hin.setHeight(zwilling2Hin.getHeight() * 0.08);
		// Bild von Zwilling2Hin einfügen
		plot.addDrawable(zwilling2Hin);

		// Zwilling 2 auf Rakete, Rückweg
		BufferedImage zwilling2RueckDatei = null; // leere Variable
		// leere Variable mit Bild füllen
		// try-catch, um Fehler beim Öffnen der Datei abzufangen
		try {
			zwilling2RueckDatei = ImageIO
					.read(new File("Zwilling2Heimweg.png"));
		} catch (IOException e) {
			e.printStackTrace();
		}
		// BoundedImage-Objekt erzeugen
		zwilling2Rueck = new BoundedImage(zwilling2RueckDatei, 0, 0);
		zwilling2Rueck.setWidth(zwilling2Rueck.getWidth() * 0.08);
		zwilling2Rueck.setHeight(zwilling2Rueck.getHeight() * 0.08);
		// Bild von Zwilling2Rück noch nicht einfügen

	}

	public void run() {

		// Zwilling 1 bewegen
		zwilling1.setXY(0, t);

		// if-Schleife für Wechsel des Bildes von Zwilling2 nach Umkehr
		if (t < zwillingsRechnung.getGesT() / 2) {
			zwilling2Hin.setXY(zwillingsRechnung.getPosZ2(t), t);
		} else if ((t >= zwillingsRechnung.getGesT() / 2) && (!rueck)) {
			plot.removeDrawable(zwilling2Hin);
			zwilling2Rueck.setXY(zwillingsRechnung.getPosZ2(t), t);
			plot.addDrawable(zwilling2Rueck);
			// boolean, damit dieser Fall genau(!) einmal gemacht wird
			rueck = true;
		} else {
			zwilling2Rueck.setXY(zwillingsRechnung.getPosZ2(t), t);
		}

		// Reise des ruhenden Zwillings (if-Schleife für Farb-Wechsel)
		if (t < zwillingsRechnung.getT1()) {
			zwilling1Data.append(0, t);
		} else if (t > zwillingsRechnung.getT1()) {
			zwilling1aData.append(0, t);
		}

		// Reise des bewegten Zwillings (if-Schleife für Farb-Wechsel)
		if (t < zwillingsRechnung.getGesT() / 2) {
			zwilling2Data.append(zwillingsRechnung.getPosZ2(t), t);
		} else if (t > zwillingsRechnung.getGesT() / 2) {
			zwilling2aData.append(zwillingsRechnung.getPosZ2(t), t);
		}

		// Gesendete Signale des ruhenden Zwillings
		for (int i = 1; i <= zwillingsRechnung.getGesT(); i++) {
			if (zwillingsRechnung.getLichtsignalZ1(t)[i - 1] != 0) {
				signal1Data.append(
						zwillingsRechnung.getLichtsignalZ1(t)[i - 1], t);
			}
		}

		// Gesendete Signale des bewegten Zwillings
		for (int j = 1; j <= zwillingsRechnung.getGesT(); j++) {
			if (zwillingsRechnung.getLichtsignalZ2(t)[j - 1] != 0) {
				signal2Data.append(
						zwillingsRechnung.getLichtsignalZ2(t)[j - 1], t);
			}
		}

		// Balkendiagramme für die Altersentwicklung: Groesse der Rechtecke wird
		// angepasst
		rechteck1.setHoehe(t);
		rechteck2.setHoehe(zwillingsRechnung.getAlterZ2SichtZ1(t));
		rechteck3.setHoehe(zwillingsRechnung.getAlterZ1SichtZ2(t));
		rechteck4.setHoehe(zwillingsRechnung.getAlterZ2SichtZ2(t));

		// Plots erneuern
		plot.repaint();
		plotZeit.repaint();
		// Ein Zeitschritt weiter
		t += dt;

		// Stop, wenn bewegter Zwilling wieder zuhause ist
		if (zwillingsRechnung.getPosZ2(t) < 0) {
			executor.shutdown();
			startButton.setEnabled(false);
			mehrerBilderButton.setEnabled(true);
			sim = false;
		}

		Toolkit.getDefaultToolkit().sync();
	}

}

/*
 * This software is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General 
 * Public License (GPL) as published by the Free Software 
 * Foundation; either version 3 of the License, 
 * or(at your option) any later version.
 *
 * This software is distributed in the hope that it will 
 * be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General 
 * Public License along with this; 
 * if not, see <http://www.gnu.org.licenses>
 */